<?php

namespace Ulmus\Api\Common;

enum HttpHeaderEnum: string
{
    # Some specially handled headers
    case A_IM = "A-IM";
    case Accept_Ch = "Accept-CH";
    case Accept_Ch_Lifetime = "Accept-CH-Lifetime";
    case Critical_Ch = "Critical-CH";
    case Content_Dpr = 'Content-DPR';
    case Content_Md5 = 'Content-MD5';
    case Correlation_Id = "Correlation_ID";
    case Dpr = "DPR";
    case Ect = "ECT";
    case ETag = "ETag";
    case Expect_CT = "Expect-CT";
    case Http2_Settings = "HTTP2-Settings";
    case IM = "IM";
    case Last_Event_Id = "Last-Event-ID";
    case Nel = "NEL";
    case P3p = "P3P";
    case Rtt = "RTT";
    case Sec_Gpc = "Sec-GPC";
    case Set_Ch_Ua = "Set-CH-UA";
    case Set_Ch_Ua_Arch = "Set-CH-UA-Arch";
    case Set_Ch_Ua_Bitness = "Set-CH-UA-Bitness";
    case Set_Ch_Ua_Full_Version = "Set-CH-UA-Full-Version";
    case Set_Ch_Ua_Full_Version_List = "Set-CH-UA-Full-Version-List";
    case Set_Ch_Ua_Mobile = "Set-CH-UA-Mobile";
    case Set_Ch_Ua_Model = "Set-CH-UA-Model";
    case Set_Ch_Ua_Platform = "Set-CH-UA-Platform";
    case Set_Ch_Ua_Platform_Version = "Set-CH-UA-Platform-Version";
    case Sourcemap = "SourceMap";
    case Te = "TE";
    case Www_Authenticate = "WWW-Authenticate";
    case X_Att_DeviceId = "X-ATT-DeviceId";
    case X_Correlation_Id = "X-Correlation-ID";
    case X_Dns_Prefetch_Control = "X-DNS-Prefetch-Control";
    case X_UA_Compatible = "X-UA-Compatible";
    case X_UIDH = "X-UIDH";
    case X_Request_Id = "X_Request_ID";
    case X_Webkit_Csp  = "X-WebKit-CSP";
    case X_Xss_Protection = "X-XSS-Protection";

    public static function normalizeHeaderKey(string $from) : string
    {
        $key = str_replace(['-'], ['_'], strtolower($from));

        foreach(HttpHeaderEnum::cases() as $case) {
            if (strtolower($case->name) === $key) {
                return $case->value;
            }
        }

        return implode('-', array_map('ucfirst', explode('-', $from)));
    }

    public static function normalizeHeaderArray(array $headers) : array
    {
        return array_combine(array_map([ static::class, 'normalizeHeaderKey' ], array_keys($headers)), array_values($headers));
    }

    public static function compileHeaders(array $associativeArray) : array
    {
        $output = [];

        foreach($associativeArray as $key => $value) {
            $output[] = sprintf("%s: %s", static::normalizeHeaderKey($key), implode(', ', (array) $value));
        }

        return $output;
    }
}
