<?php

namespace Ulmus\Api\Common;

use Psr\Http\Message\MessageInterface;
use Psr\Http\Message\StreamInterface;
use Ulmus\Api\Common\HttpHeaderEnum;

class Message implements MessageInterface
{
    protected array $headers = [];

    protected string $protocolVersion = '1.1';

    protected StreamInterface $stream;

    public function getProtocolVersion() : string
    {
        return $this->protocolVersion;
    }

    public function withProtocolVersion($version) : MessageInterface
    {
        return (clone $this)->setProtocolVersion($version);
    }

    protected function setProtocolVersion(string $version) : MessageInterface
    {
        $this->protocolVersion = $version;

        return $this;
    }

    public function getHeaders() : array
    {
        return $this->headers;
    }

    public function getHeadersLines() : array
    {
        $output = [];

        foreach($this->headers as $key => $header) {
            $output[] = $header->getHeaderLine($key);
        }

        return $output;
    }

    public function hasHeader($name) : bool
    {
        return isset($this->headers[HttpHeaderEnum::normalizeHeaderKey($name)]);
    }

    public function getHeader($name) : array
    {
        $name = HttpHeaderEnum::normalizeHeaderKey($name);

        if ( ! $this->hasHeader($name) ) {
            return [];
        }

        return $this->headers[$name];
    }

    public function getHeaderLine($name) : string
    {
        $name = HttpHeaderEnum::normalizeHeaderKey($name);

        if ( ! $this->hasHeader($name) ) {
            return "";
        }

        return sprintf("%s: %s", $name, implode(', ', $this->getHeader($name)));
    }

    public function withHeader($name, mixed $value): MessageInterface
    {
        return (clone $this)->setHeaders(array_merge_recursive($this->headers, [ HttpHeaderEnum::normalizeHeaderKey($name) => $value ]));
    }

    public function withAddedHeader($name, mixed $value) : MessageInterface
    {
        return (clone $this)->setHeaders([ HttpHeaderEnum::normalizeHeaderKey($name) => $value ] + $this->headers);
    }

    public function withoutHeader($name) : MessageInterface
    {
        return (clone $this)->unsetHeader($name);
    }

    protected function setHeaders(array $headers) : MessageInterface
    {
        $this->headers = HttpHeaderEnum::normalizeHeaderArray($headers);

        foreach($this->headers as & $item) {
            $item = is_array($item) ? $item : [ $item ];
        }

        return $this;
    }

    protected function unsetHeader(string $name) : MessageInterface
    {
        unset($this->headers[HttpHeaderEnum::normalizeHeaderKey($name)]);

        return $this;
    }

    public function getBody() : StreamInterface
    {
        return $this->stream;
    }

    public function withBody(StreamInterface $body): MessageInterface
    {
        return (clone $this)->setBody($body);
    }

    protected function setBody(StreamInterface $stream) : MessageInterface
    {
        $this->stream = $stream;

        return $this;
    }
}