<?php

namespace Ulmus\Api\Transport;

use League\OAuth2\Client\Provider\GenericProvider;
use Ulmus\Api\Common\AuthenticationEnum;

class CurlAuthorization
{
    public array $headers = [];

    public array $options = [];

    public function fromAuthenticationEnum(AuthenticationEnum $auth, #[\SensitiveParameter] array $configuration,) : static
    {
        switch($auth) {
            case AuthenticationEnum::Bearer:
                $this->bearer($configuration['token']);
                break;

            case AuthenticationEnum::Key:
                $this->key($configuration['token']);
                break;

            case AuthenticationEnum::Token:
                $this->token($configuration['token']);
                break;

            case AuthenticationEnum::Basic:
                $this->basic($configuration['username'], $configuration['password']);
                break;

            case AuthenticationEnum::Ntlm:
                $this->ntlm($configuration['username'], $configuration['password']);
                break;

            case AuthenticationEnum::Negotiate:
                $this->negotiate($configuration['username'], $configuration['password']);
                break;

            case AuthenticationEnum::OAuth2:
                $this->oauth2($configuration['grant'], $configuration['oauth_token_url'], $configuration['redirect_uri'], $configuration['client_id'], $configuration['client_secret'], $configuration['scope']);
                break;
        }

        return $this;
    }

    public function bearer(#[\SensitiveParameter] string $token) : void
    {
        $this->options[CURLOPT_HTTPAUTH] = CURLAUTH_BEARER;
        $this->options[CURLOPT_XOAUTH2_BEARER] = $token;
    }

    public function apikey(#[\SensitiveParameter] string $token) : void
    {
        $this->customToken('apikey', $token);
    }

    public function key(#[\SensitiveParameter] string $token) : void
    {
        $this->customToken('key', $token);
    }

    public function token(#[\SensitiveParameter] string $token) : void
    {
        $this->customToken('token', $token);
    }

    public function customToken(string $key, #[\SensitiveParameter] string $token) : void
    {
        $this->headers['Authorization'] = "$key {$token}";
    }

    public function basic(string $username, #[\SensitiveParameter] string $password) : void
    {
        $this->userpass(CURLAUTH_BASIC, $username, $password);
    }

    public function digest(string $username, #[\SensitiveParameter] string $password) : void
    {
        $this->userpass(CURLAUTH_DIGEST, $username, $password);
    }

    public function ntlm(string $username, #[\SensitiveParameter] string $password) : void
    {
        $this->userpass(CURLAUTH_NTLM, $username, $password);
    }

    public function negotiate(string $username, #[\SensitiveParameter] string $password) : void
    {
        $this->userpass(CURLAUTH_NEGOTIATE, $username, $password);
    }

    public function oauth2(Oauth2GrantTypeEnum $grant, string $oauthTokenUrl, string $redirectUri, string $clientId, string $clientSecret, string $scope) : void
    {
        #return;

        # TMP !
        if (false === ($_SESSION['grics_token'] ?? false)) {
            # IF NO TOKEN OR TIMEOUT
            $provider = new GenericProvider([
                'clientId'                => $clientId,
                'clientSecret'            => $clientSecret,
                'redirectUri'             => $redirectUri,
                'urlAccessToken'          => $oauthTokenUrl,
                'urlAuthorize'            => 'https://service.example.com/authorize',
                'urlResourceOwnerDetails' => 'https://service.example.com/resource',
                'scopes'                  => $scope,
            ]);

            try {
                $accessToken = $provider->getAccessToken($grant->value);
                $_SESSION['grics_token'] = $accessToken->getToken();

               # var_dump($accessToken); die();

                # KEEP TOKEN IN APCU !
            } catch (\League\OAuth2\Client\Provider\Exception\IdentityProviderException $e) {
                // Failed to get the access token
                exit($e->getMessage());
            }
        }


        $this->bearer($_SESSION['grics_token']);

        #var_dump($_SESSION['grics_token']);die();
        # SWITCH TO BEARER AUTH TYPE, SET TOKEN FROM APCU IF STILL VALID
    }

    protected function userpass(int $authType, string $username, #[\SensitiveParameter] $password) : void
    {
        $this->options[CURLOPT_HTTPAUTH] = $authType;
        $this->options[CURLOPT_USERNAME] = $username;
        $this->options[CURLOPT_PASSWORD] = $password;
    }
}